#Praat script draw_sound_f0_&_momel.praat

#author:  Daniel Hirst <daniel.hirst@lpl-aix.fr>
#version:	2011-11-02

form Draw sound, f0 and momel
	comment Picture size (in cms)
	positive width 25
	positive height 15
	comment Extract
	real start 0
	real end 0 (= all)
	positive pitch_floor 60
	real pitch_ceiling 0 (= use automatic)
	word Pitch_colour Green
	word Spline_colour Red
	word Target_colour Black
	positive Target_size 3 (=mm)
endform

debug=0

cms = 1/2.54
nSounds = numberOfSelected("Sound")
nPitchTiers = numberOfSelected("PitchTier")

if nSounds *nPitchTiers = 0
	pause Select one Sound and one PitchTier
endif

mySound = selected("Sound")
name$ = selected$("Sound")
myPitchTier = selected("PitchTier")

select mySound
duration = Get total duration

if end=0
	end = duration
endif

if pitch_ceiling = 0
	To Pitch... 0.01 60 700
	q25 = Get quantile... 0 0 0.25 Hertz
	q75 = Get quantile... 0 0 0.75 Hertz
	pitch_floor = q25 * 0.75
	pitch_ceiling = q75 *2
	Remove
endif
select mySound
myPitch = To Pitch... 0.01 pitch_floor pitch_ceiling
min_f0 = Get minimum... 0 0 Hertz None
max_f0 = Get maximum... 0 0 Hertz None

select myPitchTier
nTargets = Get number of points
for iTarget to nTargets
	hz = Get value at index... iTarget
	if hz < min_f0
		min_f0 = hz
	elsif hz > max_f0
		max_f0 = hz
	endif
endfor

			### Draw figure ###

Erase all
Select outer viewport... 0 width*cms 0 height*cms*0.8

Font size... 18
Solid line

	#Draw Pitch#

'pitch_colour$'
Line width... 2
min_f0 = floor(min_f0/25)*25
max_f0 = ceiling(max_f0/25)*25
select myPitch
Draw... start end min_f0 max_f0 no

	#Draw Quadratic Spline#

select myPitchTier
myInterpolated = Copy... 'name$'
Interpolate quadratically... 10 Hz
myQSpline = To Pitch... 0.01 min_f0 max_f0

'spline_colour$'
Draw... start end min_f0 max_f0 no

	#Draw Targets#

'target_colour$'
Line width... 1
select myPitchTier

for iTarget to nTargets
	time = Get time from index... iTarget
	hz = Get value at index... iTarget

if debug
	pause 'time:3' 'hz:0'
endif

	if time > start and time < end
		Draw circle (mm)... time hz target_size
	endif
endfor

Marks left every... 1 50 yes yes yes
Text left... yes f_0 (Hz)

	#Draw Sound#

select mySound
Select outer viewport... 0 width*cms height*cms*0.6 height*cms
Line width... 1
Draw... start end 0 0 no curve

#Add title etc.
Select outer viewport... 0 width*cms 0 height*cms
Text top... yes 'name$' 
Marks bottom every... 1 1 yes yes no
Text bottom... yes time (secs.) ['start:3'..'end:3']

select myPitch
plus myInterpolated
plus myQSpline
Remove
select mySound
plus myPitchTier

#versionhistory
#2011-11-02		First version


